/*
 * Copyright (c) 2016-2020, Sonos, Inc.
 *
 * SPDX-License-Identifier:     GPL-2.0
 *
 */

#include <linux/gpio.h>
#include "blackbox.h"
#include "micctl.h"
#include "gpio_mics.h"
#include <linux/of_gpio.h>

struct gpio gpio_mic_pin;
struct gpio gpio_mic_clk;

struct adc_ops gpio_mics_ops = (struct adc_ops){
	.mute	= gpio_mics_mute
};

int gpio_mics_mute(struct adc_ops *ref, int on)
{
	(void)ref;
	if (gpio_is_valid(gpio_mic_clk.gpio)) {
		gpio_set_value(gpio_mic_clk.gpio, !((on ^ 1) & 0x01));
	}
	gpio_set_value(gpio_mic_pin.gpio, (on ^ 1) & 0x01);
	return 0;
}

int gpio_mics_init(void)
{
	struct device_node *np = of_find_node_by_name(NULL, "sonos-gpio-micctl");

	if (!np) {
		bb_log(BB_MOD_MICCTL, BB_LVL_ERR, "No sonos-gpio-micctl entry in the dtb");
		return -1;
	}

	if (of_property_read_bool(np, "no-mics-product")) {
		return -ENODEV;
	}

	gpio_mic_pin.gpio = of_get_named_gpio(np, "micctl-gpio", 0);
	gpio_mic_pin.label = "mic_ADC_reset";

	if (gpio_request_one(gpio_mic_pin.gpio, gpio_mic_pin.flags, gpio_mic_pin.label)) {
		bb_log(BB_MOD_AMPCTL, BB_LVL_ERR, "Failed to request mic ADC reset.");
		return -1;
	}

	gpio_mic_clk.gpio = of_get_named_gpio(np, "micctl-clk-en", 0);
	if (gpio_is_valid(gpio_mic_clk.gpio)) {
		bb_log(BB_MOD_MICCTL, BB_LVL_DEBUG, "micctl-clk-en in the dtb - (%d)", gpio_mic_clk.gpio);
		gpio_mic_clk.label = "mic_clock_enable";
		if (gpio_request_one(gpio_mic_clk.gpio, gpio_mic_clk.flags, gpio_mic_clk.label)) {
			bb_log(BB_MOD_AMPCTL, BB_LVL_ERR, "Failed to request mic clock enable.");
			return -1;
		}
	} else {
		bb_log(BB_MOD_MICCTL, BB_LVL_DEBUG, "no micctl-clk-en in the dtb");
	}

	micctl_register_callbacks(&gpio_mics_ops);

	bb_log(BB_MOD_MICCTL, BB_LVL_INFO, "gpio_mics micctl driver initialized and callbacks registered");
	return 0;
}

void gpio_mics_exit(void)
{
	gpio_free(gpio_mic_pin.gpio);
}
